@@ -9,11 +9,12 @@
 
 use crate::buffer::BufferAccess;
 use crate::command_buffer::DynamicState;
-use crate::descriptor::DescriptorSet;
+use crate::descriptor_set::DescriptorSetWithOffsets;
 use crate::pipeline::input_assembly::IndexType;
 use crate::pipeline::ComputePipelineAbstract;
 use crate::pipeline::GraphicsPipelineAbstract;
-use crate::vk;
+use crate::pipeline::PipelineBindPoint;
+use crate::DeviceSize;
 use crate::VulkanObject;
 use smallvec::SmallVec;
 use std::ops::Range;
@@ -27,24 +28,24 @@ pub struct StateCacher {
     // The dynamic state to synchronize with `CmdSetState`.
     dynamic_state: DynamicState,
     // The compute pipeline currently bound. 0 if nothing bound.
-    compute_pipeline: vk::Pipeline,
+    compute_pipeline: ash::vk::Pipeline,
     // The graphics pipeline currently bound. 0 if nothing bound.
-    graphics_pipeline: vk::Pipeline,
+    graphics_pipeline: ash::vk::Pipeline,
     // The descriptor sets for the compute pipeline.
-    compute_descriptor_sets: SmallVec<[(vk::DescriptorSet, SmallVec<[u32; 32]>); 12]>,
+    compute_descriptor_sets: SmallVec<[(ash::vk::DescriptorSet, SmallVec<[u32; 32]>); 12]>,
     // The descriptor sets for the graphics pipeline.
-    graphics_descriptor_sets: SmallVec<[(vk::DescriptorSet, SmallVec<[u32; 32]>); 12]>,
+    graphics_descriptor_sets: SmallVec<[(ash::vk::DescriptorSet, SmallVec<[u32; 32]>); 12]>,
     // If the user starts comparing descriptor sets, but drops the helper struct in the middle of
     // the processing then we will end up in a weird state. This bool is true when we start
     // comparing sets, and is set to false when we end up comparing. If it was true when we start
     // comparing, we know that something bad happened and we flush the cache.
     poisoned_descriptor_sets: bool,
     // The vertex buffers currently bound.
-    vertex_buffers: SmallVec<[(vk::Buffer, vk::DeviceSize); 12]>,
+    vertex_buffers: SmallVec<[(ash::vk::Buffer, DeviceSize); 12]>,
     // Same as `poisoned_descriptor_sets` but for vertex buffers.
     poisoned_vertex_buffers: bool,
     // The index buffer, offset, and index type currently bound. `None` if nothing bound.
-    index_buffer: Option<(vk::Buffer, usize, IndexType)>,
+    index_buffer: Option<(ash::vk::Buffer, DeviceSize, IndexType)>,
 }
 
 /// Outcome of an operation.
@@ -62,8 +63,8 @@ impl StateCacher {
     pub fn new() -> StateCacher {
         StateCacher {
             dynamic_state: DynamicState::none(),
-            compute_pipeline: 0,
-            graphics_pipeline: 0,
+            compute_pipeline: ash::vk::Pipeline::null(),
+            graphics_pipeline: ash::vk::Pipeline::null(),
             compute_descriptor_sets: SmallVec::new(),
             graphics_descriptor_sets: SmallVec::new(),
             poisoned_descriptor_sets: false,
@@ -78,8 +79,8 @@ impl StateCacher {
     #[inline]
     pub fn invalidate(&mut self) {
         self.dynamic_state = DynamicState::none();
-        self.compute_pipeline = 0;
-        self.graphics_pipeline = 0;
+        self.compute_pipeline = ash::vk::Pipeline::null();
+        self.graphics_pipeline = ash::vk::Pipeline::null();
         self.compute_descriptor_sets = SmallVec::new();
         self.graphics_descriptor_sets = SmallVec::new();
         self.vertex_buffers = SmallVec::new();
@@ -125,7 +126,10 @@ impl StateCacher {
     /// This process also updates the state cacher. The state cacher assumes that the state
     /// changes are going to be performed after the `compare` function returns.
     #[inline]
-    pub fn bind_descriptor_sets(&mut self, graphics: bool) -> StateCacherDescriptorSets {
+    pub fn bind_descriptor_sets(
+        &mut self,
+        pipeline_bind_point: PipelineBindPoint,
+    ) -> StateCacherDescriptorSets {
         if self.poisoned_descriptor_sets {
             self.compute_descriptor_sets = SmallVec::new();
             self.graphics_descriptor_sets = SmallVec::new();
@@ -135,10 +139,9 @@ impl StateCacher {
 
         StateCacherDescriptorSets {
             poisoned: &mut self.poisoned_descriptor_sets,
-            state: if graphics {
-                &mut self.graphics_descriptor_sets
-            } else {
-                &mut self.compute_descriptor_sets
+            state: match pipeline_bind_point {
+                PipelineBindPoint::Compute => &mut self.compute_descriptor_sets,
+                PipelineBindPoint::Graphics => &mut self.graphics_descriptor_sets,
             },
             offset: 0,
             found_diff: None,
@@ -241,7 +244,7 @@ pub struct StateCacherDescriptorSets<'s> {
     // Reference to the parent's `poisoned_descriptor_sets`.
     poisoned: &'s mut bool,
     // Reference to the descriptor sets list to compare to.
-    state: &'s mut SmallVec<[(vk::DescriptorSet, SmallVec<[u32; 32]>); 12]>,
+    state: &'s mut SmallVec<[(ash::vk::DescriptorSet, SmallVec<[u32; 32]>); 12]>,
     // Next offset within the list to compare to.
     offset: usize,
     // Contains the return value of `compare`.
@@ -251,21 +254,20 @@ pub struct StateCacherDescriptorSets<'s> {
 impl<'s> StateCacherDescriptorSets<'s> {
     /// Adds a descriptor set to the list to compare.
     #[inline]
-    pub fn add<S>(&mut self, set: &S, dynamic_offsets: &SmallVec<[u32; 32]>)
-    where
-        S: ?Sized + DescriptorSet,
-    {
-        let raw = set.inner().internal_object();
+    pub fn add(&mut self, descriptor_set: &DescriptorSetWithOffsets) {
+        let (descriptor_set, dynamic_offsets) = descriptor_set.as_ref();
+        let raw = descriptor_set.inner().internal_object();
+        let dynamic_offsets = dynamic_offsets.iter().copied().collect();
 
-        if self.offset < self.state.len() {
-            if (&self.state[self.offset].0, &self.state[self.offset].1) == (&raw, dynamic_offsets) {
+        if let Some(state) = self.state.get_mut(self.offset) {
+            if (&state.0, &state.1) == (&raw, &dynamic_offsets) {
                 self.offset += 1;
                 return;
             }
 
-            self.state[self.offset] = (raw, dynamic_offsets.clone());
+            *state = (raw, dynamic_offsets);
         } else {
-            self.state.push((raw, dynamic_offsets.clone()));
+            self.state.push((raw, dynamic_offsets));
         }
 
         if self.found_diff.is_none() {
@@ -295,7 +297,7 @@ pub struct StateCacherVertexBuffers<'s> {
     // Reference to the parent's `poisoned_vertex_buffers`.
     poisoned: &'s mut bool,
     // Reference to the vertex buffers list to compare to.
-    state: &'s mut SmallVec<[(vk::Buffer, vk::DeviceSize); 12]>,
+    state: &'s mut SmallVec<[(ash::vk::Buffer, DeviceSize); 12]>,
     // Next offset within the list to compare to.
     offset: usize,
     // Contains the offset of the first vertex buffer that differs.
@@ -314,7 +316,7 @@ impl<'s> StateCacherVertexBuffers<'s> {
         let raw = {
             let inner = buffer.inner();
             let raw = inner.buffer.internal_object();
-            let offset = inner.offset as vk::DeviceSize;
+            let offset = inner.offset;
             (raw, offset)
         };
 
