@@ -7,7 +7,6 @@
 // notice may not be copied, modified, or distributed except
 // according to those terms.
 
-use crate::buffer::BufferAccess;
 use crate::device::Device;
 use crate::format::ClearValue;
 use crate::format::Format;
@@ -23,6 +22,7 @@ use crate::image::ImageDimensions;
 use crate::image::ImageInner;
 use crate::image::ImageLayout;
 use crate::image::ImageUsage;
+use crate::image::SampleCount;
 use crate::memory::pool::AllocFromRequirementsFilter;
 use crate::memory::pool::AllocLayout;
 use crate::memory::pool::MappingRequirement;
@@ -104,7 +104,14 @@ impl AttachmentImage {
         dimensions: [u32; 2],
         format: Format,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
-        AttachmentImage::new_impl(device, dimensions, format, ImageUsage::none(), 1)
+        AttachmentImage::new_impl(
+            device,
+            dimensions,
+            1,
+            format,
+            ImageUsage::none(),
+            SampleCount::Sample1,
+        )
     }
 
     /// Same as `new`, but creates an image that can be used as an input attachment.
@@ -121,7 +128,14 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, 1)
+        AttachmentImage::new_impl(
+            device,
+            dimensions,
+            1,
+            format,
+            base_usage,
+            SampleCount::Sample1,
+        )
     }
 
     /// Same as `new`, but creates a multisampled image.
@@ -132,10 +146,10 @@ impl AttachmentImage {
     pub fn multisampled(
         device: Arc<Device>,
         dimensions: [u32; 2],
-        samples: u32,
+        samples: SampleCount,
         format: Format,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
-        AttachmentImage::new_impl(device, dimensions, format, ImageUsage::none(), samples)
+        AttachmentImage::new_impl(device, dimensions, 1, format, ImageUsage::none(), samples)
     }
 
     /// Same as `multisampled`, but creates an image that can be used as an input attachment.
@@ -145,7 +159,7 @@ impl AttachmentImage {
     pub fn multisampled_input_attachment(
         device: Arc<Device>,
         dimensions: [u32; 2],
-        samples: u32,
+        samples: SampleCount,
         format: Format,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
         let base_usage = ImageUsage {
@@ -153,7 +167,7 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, samples)
+        AttachmentImage::new_impl(device, dimensions, 1, format, base_usage, samples)
     }
 
     /// Same as `new`, but lets you specify additional usages.
@@ -168,24 +182,38 @@ impl AttachmentImage {
         format: Format,
         usage: ImageUsage,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
-        AttachmentImage::new_impl(device, dimensions, format, usage, 1)
+        AttachmentImage::new_impl(device, dimensions, 1, format, usage, SampleCount::Sample1)
     }
 
     /// Same as `with_usage`, but creates a multisampled image.
     ///
     /// > **Note**: You can also use this function and pass `1` for the number of samples if you
     /// > want a regular image.
-    ///
-    /// > **Note**: This function is just a convenient shortcut for `multisampled_with_usage`.
     #[inline]
     pub fn multisampled_with_usage(
         device: Arc<Device>,
         dimensions: [u32; 2],
-        samples: u32,
+        samples: SampleCount,
         format: Format,
         usage: ImageUsage,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
-        AttachmentImage::new_impl(device, dimensions, format, usage, samples)
+        AttachmentImage::new_impl(device, dimensions, 1, format, usage, samples)
+    }
+
+    /// Same as `multisampled_with_usage`, but creates an image with multiple layers.
+    ///
+    /// > **Note**: You can also use this function and pass `1` for the number of layers if you
+    /// > want a regular image.
+    #[inline]
+    pub fn multisampled_with_usage_with_layers(
+        device: Arc<Device>,
+        dimensions: [u32; 2],
+        array_layers: u32,
+        samples: SampleCount,
+        format: Format,
+        usage: ImageUsage,
+    ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
+        AttachmentImage::new_impl(device, dimensions, array_layers, format, usage, samples)
     }
 
     /// Same as `new`, except that the image can later be sampled.
@@ -202,7 +230,14 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, 1)
+        AttachmentImage::new_impl(
+            device,
+            dimensions,
+            1,
+            format,
+            base_usage,
+            SampleCount::Sample1,
+        )
     }
 
     /// Same as `sampled`, except that the image can be used as an input attachment.
@@ -220,7 +255,14 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, 1)
+        AttachmentImage::new_impl(
+            device,
+            dimensions,
+            1,
+            format,
+            base_usage,
+            SampleCount::Sample1,
+        )
     }
 
     /// Same as `sampled`, but creates a multisampled image.
@@ -233,7 +275,7 @@ impl AttachmentImage {
     pub fn sampled_multisampled(
         device: Arc<Device>,
         dimensions: [u32; 2],
-        samples: u32,
+        samples: SampleCount,
         format: Format,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
         let base_usage = ImageUsage {
@@ -241,7 +283,7 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, samples)
+        AttachmentImage::new_impl(device, dimensions, 1, format, base_usage, samples)
     }
 
     /// Same as `sampled_multisampled`, but creates an image that can be used as an input
@@ -252,7 +294,7 @@ impl AttachmentImage {
     pub fn sampled_multisampled_input_attachment(
         device: Arc<Device>,
         dimensions: [u32; 2],
-        samples: u32,
+        samples: SampleCount,
         format: Format,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
         let base_usage = ImageUsage {
@@ -261,7 +303,7 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, samples)
+        AttachmentImage::new_impl(device, dimensions, 1, format, base_usage, samples)
     }
 
     /// Same as `new`, except that the image will be transient.
@@ -281,7 +323,14 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, 1)
+        AttachmentImage::new_impl(
+            device,
+            dimensions,
+            1,
+            format,
+            base_usage,
+            SampleCount::Sample1,
+        )
     }
 
     /// Same as `transient`, except that the image can be used as an input attachment.
@@ -299,7 +348,14 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, 1)
+        AttachmentImage::new_impl(
+            device,
+            dimensions,
+            1,
+            format,
+            base_usage,
+            SampleCount::Sample1,
+        )
     }
 
     /// Same as `transient`, but creates a multisampled image.
@@ -312,7 +368,7 @@ impl AttachmentImage {
     pub fn transient_multisampled(
         device: Arc<Device>,
         dimensions: [u32; 2],
-        samples: u32,
+        samples: SampleCount,
         format: Format,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
         let base_usage = ImageUsage {
@@ -320,7 +376,7 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, samples)
+        AttachmentImage::new_impl(device, dimensions, 1, format, base_usage, samples)
     }
 
     /// Same as `transient_multisampled`, but creates an image that can be used as an input
@@ -331,7 +387,7 @@ impl AttachmentImage {
     pub fn transient_multisampled_input_attachment(
         device: Arc<Device>,
         dimensions: [u32; 2],
-        samples: u32,
+        samples: SampleCount,
         format: Format,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
         let base_usage = ImageUsage {
@@ -340,16 +396,17 @@ impl AttachmentImage {
             ..ImageUsage::none()
         };
 
-        AttachmentImage::new_impl(device, dimensions, format, base_usage, samples)
+        AttachmentImage::new_impl(device, dimensions, 1, format, base_usage, samples)
     }
 
     // All constructors dispatch to this one.
     fn new_impl(
         device: Arc<Device>,
         dimensions: [u32; 2],
+        array_layers: u32,
         format: Format,
         base_usage: ImageUsage,
-        samples: u32,
+        samples: SampleCount,
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
         // TODO: check dimensions against the max_framebuffer_width/height/layers limits
 
@@ -371,7 +428,7 @@ impl AttachmentImage {
             let dims = ImageDimensions::Dim2d {
                 width: dimensions[0],
                 height: dimensions[1],
-                array_layers: 1,
+                array_layers,
             };
 
             UnsafeImage::new(
@@ -463,23 +520,18 @@ unsafe impl<A> ImageAccess for AttachmentImage<A> {
         })
     }
 
-    #[inline]
-    fn conflicts_buffer(&self, other: &dyn BufferAccess) -> bool {
-        false
-    }
-
-    #[inline]
-    fn conflicts_image(&self, other: &dyn ImageAccess) -> bool {
-        self.conflict_key() == other.conflict_key()
-    }
-
     #[inline]
     fn conflict_key(&self) -> u64 {
         self.image.key()
     }
 
     #[inline]
-    fn try_gpu_lock(&self, _: bool, expected_layout: ImageLayout) -> Result<(), AccessError> {
+    fn try_gpu_lock(
+        &self,
+        _: bool,
+        uninitialized_safe: bool,
+        expected_layout: ImageLayout,
+    ) -> Result<(), AccessError> {
         if expected_layout != self.attachment_layout && expected_layout != ImageLayout::Undefined {
             if self.initialized.load(Ordering::SeqCst) {
                 return Err(AccessError::UnexpectedImageLayout {
@@ -494,7 +546,7 @@ unsafe impl<A> ImageAccess for AttachmentImage<A> {
             }
         }
 
-        if expected_layout != ImageLayout::Undefined {
+        if !uninitialized_safe && expected_layout != ImageLayout::Undefined {
             if !self.initialized.load(Ordering::SeqCst) {
                 return Err(AccessError::ImageNotInitialized {
                     requested: expected_layout,
