@@ -16,6 +16,7 @@ use crate::command_buffer::CommandBufferExecFuture;
 use crate::command_buffer::CommandBufferUsage;
 use crate::command_buffer::PrimaryAutoCommandBuffer;
 use crate::command_buffer::PrimaryCommandBuffer;
+use crate::device::physical::QueueFamily;
 use crate::device::Device;
 use crate::device::Queue;
 use crate::format::Format;
@@ -31,7 +32,7 @@ use crate::image::ImageInner;
 use crate::image::ImageLayout;
 use crate::image::ImageUsage;
 use crate::image::MipmapsCount;
-use crate::instance::QueueFamily;
+use crate::image::SampleCount;
 use crate::memory::pool::AllocFromRequirementsFilter;
 use crate::memory::pool::AllocLayout;
 use crate::memory::pool::MappingRequirement;
@@ -267,7 +268,7 @@ impl ImmutableImage {
                 format,
                 flags,
                 dimensions,
-                1,
+                SampleCount::Sample1,
                 mipmaps,
                 sharing,
                 false,
@@ -473,16 +474,6 @@ unsafe impl<A> ImageAccess for ImmutableImage<A> {
         })
     }
 
-    #[inline]
-    fn conflicts_buffer(&self, other: &dyn BufferAccess) -> bool {
-        false
-    }
-
-    #[inline]
-    fn conflicts_image(&self, other: &dyn ImageAccess) -> bool {
-        self.conflict_key() == other.conflict_key() // TODO:
-    }
-
     #[inline]
     fn conflict_key(&self) -> u64 {
         self.image.key()
@@ -492,6 +483,7 @@ unsafe impl<A> ImageAccess for ImmutableImage<A> {
     fn try_gpu_lock(
         &self,
         exclusive_access: bool,
+        uninitialized_safe: bool,
         expected_layout: ImageLayout,
     ) -> Result<(), AccessError> {
         if expected_layout != self.layout && expected_layout != ImageLayout::Undefined {
@@ -559,18 +551,6 @@ unsafe impl ImageAccess for SubImage {
         None
     }
 
-    #[inline]
-    fn conflicts_buffer(&self, other: &dyn BufferAccess) -> bool {
-        false
-    }
-
-    #[inline]
-    fn conflicts_image(&self, other: &dyn ImageAccess) -> bool {
-        self.conflict_key() == other.conflict_key()
-            && self.current_miplevels_access() == other.current_miplevels_access()
-            && self.current_layer_levels_access() == other.current_layer_levels_access()
-    }
-
     fn current_miplevels_access(&self) -> std::ops::Range<u32> {
         self.mip_levels_access.clone()
     }
@@ -588,6 +568,7 @@ unsafe impl ImageAccess for SubImage {
     fn try_gpu_lock(
         &self,
         exclusive_access: bool,
+        uninitialized_safe: bool,
         expected_layout: ImageLayout,
     ) -> Result<(), AccessError> {
         if expected_layout != self.layout && expected_layout != ImageLayout::Undefined {
@@ -648,23 +629,18 @@ unsafe impl<A> ImageAccess for ImmutableImageInitialization<A> {
         None
     }
 
-    #[inline]
-    fn conflicts_buffer(&self, other: &dyn BufferAccess) -> bool {
-        false
-    }
-
-    #[inline]
-    fn conflicts_image(&self, other: &dyn ImageAccess) -> bool {
-        self.conflict_key() == other.conflict_key() // TODO:
-    }
-
     #[inline]
     fn conflict_key(&self) -> u64 {
         self.image.image.key()
     }
 
     #[inline]
-    fn try_gpu_lock(&self, _: bool, expected_layout: ImageLayout) -> Result<(), AccessError> {
+    fn try_gpu_lock(
+        &self,
+        _: bool,
+        uninitialized_safe: bool,
+        expected_layout: ImageLayout,
+    ) -> Result<(), AccessError> {
         if expected_layout != ImageLayout::Undefined {
             return Err(AccessError::UnexpectedImageLayout {
                 requested: expected_layout,
