@@ -7,35 +7,34 @@
 // notice may not be copied, modified, or distributed except
 // according to those terms.
 
+use crate::check_errors;
+use crate::device::physical::MemoryType;
+use crate::device::Device;
+use crate::device::DeviceOwned;
+use crate::memory::Content;
+use crate::memory::DedicatedAlloc;
+use crate::memory::ExternalMemoryHandleType;
+use crate::DeviceSize;
+use crate::Error;
+use crate::OomError;
+use crate::Version;
+use crate::VulkanObject;
 use std::error;
 use std::fmt;
+#[cfg(target_os = "linux")]
+use std::fs::File;
 use std::marker::PhantomData;
 use std::mem::MaybeUninit;
 use std::ops::Deref;
 use std::ops::DerefMut;
 use std::ops::Range;
 use std::os::raw::c_void;
+#[cfg(target_os = "linux")]
+use std::os::unix::io::{FromRawFd, IntoRawFd};
 use std::ptr;
 use std::sync::Arc;
 use std::sync::Mutex;
 
-#[cfg(target_os = "linux")]
-use std::fs::File;
-#[cfg(target_os = "linux")]
-use std::os::unix::io::{FromRawFd, IntoRawFd};
-
-use crate::check_errors;
-use crate::device::Device;
-use crate::device::DeviceOwned;
-use crate::instance::MemoryType;
-use crate::memory::Content;
-use crate::memory::DedicatedAlloc;
-use crate::memory::ExternalMemoryHandleType;
-use crate::vk;
-use crate::Error;
-use crate::OomError;
-use crate::VulkanObject;
-
 #[repr(C)]
 pub struct BaseOutStructure {
     pub s_type: i32,
@@ -56,9 +55,9 @@ pub(crate) unsafe fn ptr_chain_iter<T>(ptr: &mut T) -> impl Iterator<Item = *mut
 }
 
 pub unsafe trait ExtendsMemoryAllocateInfo {}
-unsafe impl ExtendsMemoryAllocateInfo for vk::MemoryDedicatedAllocateInfoKHR {}
-unsafe impl ExtendsMemoryAllocateInfo for vk::ExportMemoryAllocateInfo {}
-unsafe impl ExtendsMemoryAllocateInfo for vk::ImportMemoryFdInfoKHR {}
+unsafe impl ExtendsMemoryAllocateInfo for ash::vk::MemoryDedicatedAllocateInfoKHR {}
+unsafe impl ExtendsMemoryAllocateInfo for ash::vk::ExportMemoryAllocateInfo {}
+unsafe impl ExtendsMemoryAllocateInfo for ash::vk::ImportMemoryFdInfoKHR {}
 
 /// Represents memory that has been allocated.
 ///
@@ -76,9 +75,9 @@ unsafe impl ExtendsMemoryAllocateInfo for vk::ImportMemoryFdInfoKHR {}
 /// let memory = DeviceMemory::alloc(device.clone(), mem_ty, 1024).unwrap();
 /// ```
 pub struct DeviceMemory {
-    memory: vk::DeviceMemory,
+    memory: ash::vk::DeviceMemory,
     device: Arc<Device>,
-    size: usize,
+    size: DeviceSize,
     memory_type_index: u32,
     handle_types: ExternalMemoryHandleType,
     mapped: Mutex<bool>,
@@ -99,22 +98,25 @@ pub struct DeviceMemory {
 /// ```
 pub struct DeviceMemoryBuilder<'a> {
     device: Arc<Device>,
-    allocate: vk::MemoryAllocateInfo,
-    dedicated_info: Option<vk::MemoryDedicatedAllocateInfoKHR>,
-    export_info: Option<vk::ExportMemoryAllocateInfo>,
-    import_info: Option<vk::ImportMemoryFdInfoKHR>,
+    allocate: ash::vk::MemoryAllocateInfo,
+    dedicated_info: Option<ash::vk::MemoryDedicatedAllocateInfoKHR>,
+    export_info: Option<ash::vk::ExportMemoryAllocateInfo>,
+    import_info: Option<ash::vk::ImportMemoryFdInfoKHR>,
     marker: PhantomData<&'a ()>,
 }
 
 impl<'a> DeviceMemoryBuilder<'a> {
     /// Returns a new `DeviceMemoryBuilder` given the required device, memory type and size fields.
     /// Validation of parameters is done when the builder is built.
-    pub fn new(device: Arc<Device>, memory_index: u32, size: usize) -> DeviceMemoryBuilder<'a> {
-        let allocate = vk::MemoryAllocateInfo {
-            sType: vk::STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO,
-            pNext: ptr::null(),
-            allocationSize: size as u64,
-            memoryTypeIndex: memory_index,
+    pub fn new(
+        device: Arc<Device>,
+        memory_index: u32,
+        size: DeviceSize,
+    ) -> DeviceMemoryBuilder<'a> {
+        let allocate = ash::vk::MemoryAllocateInfo {
+            allocation_size: size,
+            memory_type_index: memory_index,
+            ..Default::default()
         };
 
         DeviceMemoryBuilder {
@@ -137,18 +139,22 @@ impl<'a> DeviceMemoryBuilder<'a> {
     pub fn dedicated_info(mut self, dedicated: DedicatedAlloc<'a>) -> DeviceMemoryBuilder {
         assert!(self.dedicated_info.is_none());
 
+        if !(self.device.api_version() >= Version::V1_1
+            || self.device.enabled_extensions().khr_dedicated_allocation)
+        {
+            return self;
+        }
+
         let mut dedicated_info = match dedicated {
-            DedicatedAlloc::Buffer(buffer) => vk::MemoryDedicatedAllocateInfoKHR {
-                sType: vk::STRUCTURE_TYPE_MEMORY_DEDICATED_ALLOCATE_INFO_KHR,
-                pNext: ptr::null(),
-                image: 0,
+            DedicatedAlloc::Buffer(buffer) => ash::vk::MemoryDedicatedAllocateInfoKHR {
+                image: ash::vk::Image::null(),
                 buffer: buffer.internal_object(),
+                ..Default::default()
             },
-            DedicatedAlloc::Image(image) => vk::MemoryDedicatedAllocateInfoKHR {
-                sType: vk::STRUCTURE_TYPE_MEMORY_DEDICATED_ALLOCATE_INFO_KHR,
-                pNext: ptr::null(),
+            DedicatedAlloc::Image(image) => ash::vk::MemoryDedicatedAllocateInfoKHR {
                 image: image.internal_object(),
-                buffer: 0,
+                buffer: ash::vk::Buffer::null(),
+                ..Default::default()
             },
             DedicatedAlloc::None => return self,
         };
@@ -169,10 +175,9 @@ impl<'a> DeviceMemoryBuilder<'a> {
     ) -> DeviceMemoryBuilder<'a> {
         assert!(self.export_info.is_none());
 
-        let mut export_info = vk::ExportMemoryAllocateInfo {
-            sType: vk::STRUCTURE_TYPE_EXPORT_MEMORY_ALLOCATE_INFO,
-            pNext: ptr::null(),
-            handleTypes: handle_types.into(),
+        let mut export_info = ash::vk::ExportMemoryAllocateInfo {
+            handle_types: handle_types.into(),
+            ..Default::default()
         };
 
         self = self.push_next(&mut export_info);
@@ -193,11 +198,10 @@ impl<'a> DeviceMemoryBuilder<'a> {
     ) -> DeviceMemoryBuilder<'a> {
         assert!(self.import_info.is_none());
 
-        let mut import_info = vk::ImportMemoryFdInfoKHR {
-            sType: vk::STRUCTURE_TYPE_IMPORT_MEMORY_FD_INFO_KHR,
-            pNext: ptr::null(),
-            handleType: handle_types.into(),
+        let mut import_info = ash::vk::ImportMemoryFdInfoKHR {
+            handle_type: handle_types.into(),
             fd: fd.into_raw_fd(),
+            ..Default::default()
         };
 
         self = self.push_next(&mut import_info);
@@ -205,13 +209,31 @@ impl<'a> DeviceMemoryBuilder<'a> {
         self
     }
 
-    // Private function -- no doc comment needed!  Copied shamelessly and poorly from Ash.
+    // Private function copied shamelessly from Ash.
+    // https://github.com/MaikKlein/ash/blob/4ba8637d018fec6d6e3a90d7fa47d11c085f6b4a/generator/src/lib.rs
+    #[allow(unused_assignments)]
     fn push_next<T: ExtendsMemoryAllocateInfo>(self, next: &mut T) -> DeviceMemoryBuilder<'a> {
         unsafe {
+            // `next` here can contain a pointer chain. This means that we must correctly
+            // attach he head to the root and the tail to the rest of the chain
+            // For example:
+            //
+            // next = A -> B
+            // Before: `Root -> C -> D -> E`
+            // After: `Root -> A -> B -> C -> D -> E`
+
+            // Convert next to our ptr structure
             let next_ptr = next as *mut T as *mut BaseOutStructure;
-            let mut prev = self.allocate.pNext as *mut BaseOutStructure;
-            let last_next = ptr_chain_iter(&mut prev).last().unwrap();
-            (*last_next).p_next = next_ptr as _;
+            // Previous head (can be null)
+            let mut prev_head = self.allocate.p_next as *mut BaseOutStructure;
+            // Retrieve end of next chain
+            let last_next = ptr_chain_iter(next).last().unwrap();
+            // Set end of next chain's next to be previous head only if previous head's next'
+            if !prev_head.is_null() {
+                (*last_next).p_next = (*prev_head).p_next;
+            }
+            // Set next ptr to be first one
+            prev_head = next_ptr;
         }
 
         self
@@ -221,7 +243,7 @@ impl<'a> DeviceMemoryBuilder<'a> {
     /// is returned if the requested allocation is too large or if the total number of allocations
     /// would exceed per-device limits.
     pub fn build(self) -> Result<Arc<DeviceMemory>, DeviceMemoryAllocError> {
-        if self.allocate.allocationSize == 0 {
+        if self.allocate.allocation_size == 0 {
             return Err(DeviceMemoryAllocError::InvalidSize)?;
         }
 
@@ -232,7 +254,7 @@ impl<'a> DeviceMemoryBuilder<'a> {
         let memory_type = self
             .device
             .physical_device()
-            .memory_type_by_id(self.allocate.memoryTypeIndex)
+            .memory_type_by_id(self.allocate.memory_type_index)
             .ok_or(DeviceMemoryAllocError::SpecViolation(1714))?;
 
         if self.device.physical_device().internal_object()
@@ -252,58 +274,57 @@ impl<'a> DeviceMemoryBuilder<'a> {
         // VkPhysicalDeviceMemoryProperties::memoryTypes[pAllocateInfo->memoryTypeIndex].heapIndex as
         // returned by vkGetPhysicalDeviceMemoryProperties for the VkPhysicalDevice that device was created
         // from".
-        let reported_heap_size = memory_type.heap().size() as u64;
-        if reported_heap_size != 0 && self.allocate.allocationSize > reported_heap_size {
+        let reported_heap_size = memory_type.heap().size();
+        if reported_heap_size != 0 && self.allocate.allocation_size > reported_heap_size {
             return Err(DeviceMemoryAllocError::SpecViolation(1713));
         }
 
-        let mut export_handle_bits = 0;
-        if self.dedicated_info.is_some() {
-            if !self.device.loaded_extensions().khr_dedicated_allocation {
-                return Err(DeviceMemoryAllocError::MissingExtension(
-                    "khr_dedicated_allocation",
-                ));
-            }
-        }
+        let mut export_handle_bits = ash::vk::ExternalMemoryHandleTypeFlags::empty();
 
         if self.export_info.is_some() || self.import_info.is_some() {
             // TODO: check exportFromImportedHandleTypes
             export_handle_bits = match self.export_info {
-                Some(export_info) => export_info.handleTypes,
-                None => 0,
+                Some(export_info) => export_info.handle_types,
+                None => ash::vk::ExternalMemoryHandleTypeFlags::empty(),
             };
 
             let import_handle_bits = match self.import_info {
-                Some(import_info) => import_info.handleType,
-                None => 0,
+                Some(import_info) => import_info.handle_type,
+                None => ash::vk::ExternalMemoryHandleTypeFlags::empty(),
             };
 
-            if export_handle_bits & vk::EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT != 0 {
-                if !self.device.loaded_extensions().ext_external_memory_dmabuf {
+            if !(export_handle_bits & ash::vk::ExternalMemoryHandleTypeFlags::DMA_BUF_EXT)
+                .is_empty()
+            {
+                if !self.device.enabled_extensions().ext_external_memory_dma_buf {
                     return Err(DeviceMemoryAllocError::MissingExtension(
                         "ext_external_memory_dmabuf",
                     ));
                 };
             }
 
-            if export_handle_bits & vk::EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT != 0 {
-                if !self.device.loaded_extensions().khr_external_memory_fd {
+            if !(export_handle_bits & ash::vk::ExternalMemoryHandleTypeFlags::OPAQUE_FD).is_empty()
+            {
+                if !self.device.enabled_extensions().khr_external_memory_fd {
                     return Err(DeviceMemoryAllocError::MissingExtension(
                         "khr_external_memory_fd",
                     ));
                 }
             }
 
-            if import_handle_bits & vk::EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT != 0 {
-                if !self.device.loaded_extensions().ext_external_memory_dmabuf {
+            if !(import_handle_bits & ash::vk::ExternalMemoryHandleTypeFlags::DMA_BUF_EXT)
+                .is_empty()
+            {
+                if !self.device.enabled_extensions().ext_external_memory_dma_buf {
                     return Err(DeviceMemoryAllocError::MissingExtension(
                         "ext_external_memory_dmabuf",
                     ));
                 }
             }
 
-            if import_handle_bits & vk::EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT != 0 {
-                if !self.device.loaded_extensions().khr_external_memory_fd {
+            if !(import_handle_bits & ash::vk::ExternalMemoryHandleTypeFlags::OPAQUE_FD).is_empty()
+            {
+                if !self.device.enabled_extensions().khr_external_memory_fd {
                     return Err(DeviceMemoryAllocError::MissingExtension(
                         "khr_external_memory_fd",
                     ));
@@ -319,13 +340,17 @@ impl<'a> DeviceMemoryBuilder<'a> {
                 .lock()
                 .expect("Poisoned mutex");
 
-            if *allocation_count >= physical_device.limits().max_memory_allocation_count() {
+            if *allocation_count
+                >= physical_device
+                    .properties()
+                    .max_memory_allocation_count
+            {
                 return Err(DeviceMemoryAllocError::TooManyObjects);
             }
-            let vk = self.device.pointers();
+            let fns = self.device.fns();
 
             let mut output = MaybeUninit::uninit();
-            check_errors(vk.AllocateMemory(
+            check_errors(fns.v1_0.allocate_memory(
                 self.device.internal_object(),
                 &self.allocate,
                 ptr::null(),
@@ -338,8 +363,8 @@ impl<'a> DeviceMemoryBuilder<'a> {
         Ok(Arc::new(DeviceMemory {
             memory: memory,
             device: self.device,
-            size: self.allocate.allocationSize as usize,
-            memory_type_index: self.allocate.memoryTypeIndex,
+            size: self.allocate.allocation_size,
+            memory_type_index: self.allocate.memory_type_index,
             handle_types: ExternalMemoryHandleType::from(export_handle_bits),
             mapped: Mutex::new(false),
         }))
@@ -361,7 +386,7 @@ impl DeviceMemory {
     pub fn alloc(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
     ) -> Result<DeviceMemory, DeviceMemoryAllocError> {
         let memory = DeviceMemoryBuilder::new(device, memory_type.id(), size).build()?;
         // Will never panic because we call the DeviceMemoryBuilder internally, and that only
@@ -380,7 +405,7 @@ impl DeviceMemory {
     pub fn dedicated_alloc(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
         resource: DedicatedAlloc,
     ) -> Result<DeviceMemory, DeviceMemoryAllocError> {
         let memory = DeviceMemoryBuilder::new(device, memory_type.id(), size)
@@ -403,7 +428,7 @@ impl DeviceMemory {
     pub fn alloc_and_map(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
     ) -> Result<MappedDeviceMemory, DeviceMemoryAllocError> {
         DeviceMemory::dedicated_alloc_and_map(device, memory_type, size, DedicatedAlloc::None)
     }
@@ -412,10 +437,10 @@ impl DeviceMemory {
     pub fn dedicated_alloc_and_map(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
         resource: DedicatedAlloc,
     ) -> Result<MappedDeviceMemory, DeviceMemoryAllocError> {
-        let vk = device.pointers();
+        let fns = device.fns();
 
         assert!(memory_type.is_host_visible());
         let mem = DeviceMemory::dedicated_alloc(device.clone(), memory_type, size, resource)?;
@@ -429,7 +454,7 @@ impl DeviceMemory {
     pub fn alloc_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
     ) -> Result<DeviceMemory, DeviceMemoryAllocError> {
         let memory = DeviceMemoryBuilder::new(device, memory_type.id(), size)
             .export_info(ExternalMemoryHandleType {
@@ -449,7 +474,7 @@ impl DeviceMemory {
     pub fn dedicated_alloc_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
         resource: DedicatedAlloc,
     ) -> Result<DeviceMemory, DeviceMemoryAllocError> {
         let memory = DeviceMemoryBuilder::new(device, memory_type.id(), size)
@@ -471,7 +496,7 @@ impl DeviceMemory {
     pub fn alloc_and_map_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
     ) -> Result<MappedDeviceMemory, DeviceMemoryAllocError> {
         DeviceMemory::dedicated_alloc_and_map_with_exportable_fd(
             device,
@@ -487,10 +512,10 @@ impl DeviceMemory {
     pub fn dedicated_alloc_and_map_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
-        size: usize,
+        size: DeviceSize,
         resource: DedicatedAlloc,
     ) -> Result<MappedDeviceMemory, DeviceMemoryAllocError> {
-        let vk = device.pointers();
+        let fns = device.fns();
 
         assert!(memory_type.is_host_visible());
         let mem = DeviceMemory::dedicated_alloc_with_exportable_fd(
@@ -507,16 +532,16 @@ impl DeviceMemory {
         device: Arc<Device>,
         mem: DeviceMemory,
     ) -> Result<MappedDeviceMemory, DeviceMemoryAllocError> {
-        let vk = device.pointers();
+        let fns = device.fns();
         let coherent = mem.memory_type().is_host_coherent();
         let ptr = unsafe {
             let mut output = MaybeUninit::uninit();
-            check_errors(vk.MapMemory(
+            check_errors(fns.v1_0.map_memory(
                 device.internal_object(),
                 mem.memory,
                 0,
-                mem.size as vk::DeviceSize,
-                0, /* reserved flags */
+                mem.size,
+                ash::vk::MemoryMapFlags::empty(),
                 output.as_mut_ptr(),
             ))?;
             output.assume_init()
@@ -540,7 +565,7 @@ impl DeviceMemory {
 
     /// Returns the size in bytes of that memory chunk.
     #[inline]
-    pub fn size(&self) -> usize {
+    pub fn size(&self) -> DeviceSize {
         self.size
     }
 
@@ -556,33 +581,32 @@ impl DeviceMemory {
         &self,
         handle_type: ExternalMemoryHandleType,
     ) -> Result<File, DeviceMemoryAllocError> {
-        let vk = self.device.pointers();
+        let fns = self.device.fns();
 
         // VUID-VkMemoryGetFdInfoKHR-handleType-00672: "handleType must be defined as a POSIX file
         // descriptor handle".
-        let bits = vk::ExternalMemoryHandleTypeFlags::from(handle_type);
-        if bits != vk::EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT
-            && bits != vk::EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT
+        let bits = ash::vk::ExternalMemoryHandleTypeFlags::from(handle_type);
+        if bits != ash::vk::ExternalMemoryHandleTypeFlags::DMA_BUF_EXT
+            && bits != ash::vk::ExternalMemoryHandleTypeFlags::OPAQUE_FD
         {
             return Err(DeviceMemoryAllocError::SpecViolation(672))?;
         }
 
         // VUID-VkMemoryGetFdInfoKHR-handleType-00671: "handleType must have been included in
         // VkExportMemoryAllocateInfo::handleTypes when memory was created".
-        if bits & vk::ExternalMemoryHandleTypeFlags::from(self.handle_types) == 0 {
+        if (bits & ash::vk::ExternalMemoryHandleTypeFlags::from(self.handle_types)).is_empty() {
             return Err(DeviceMemoryAllocError::SpecViolation(671))?;
         }
 
         let fd = unsafe {
-            let info = vk::MemoryGetFdInfoKHR {
-                sType: vk::STRUCTURE_TYPE_MEMORY_GET_FD_INFO_KHR,
-                pNext: ptr::null(),
+            let info = ash::vk::MemoryGetFdInfoKHR {
                 memory: self.memory,
-                handleType: handle_type.into(),
+                handle_type: handle_type.into(),
+                ..Default::default()
             };
 
             let mut output = MaybeUninit::uninit();
-            check_errors(vk.GetMemoryFdKHR(
+            check_errors(fns.khr_external_memory_fd.get_memory_fd_khr(
                 self.device.internal_object(),
                 &info,
                 output.as_mut_ptr(),
@@ -613,12 +637,10 @@ impl fmt::Debug for DeviceMemory {
 }
 
 unsafe impl VulkanObject for DeviceMemory {
-    type Object = vk::DeviceMemory;
-
-    const TYPE: vk::ObjectType = vk::OBJECT_TYPE_DEVICE_MEMORY;
+    type Object = ash::vk::DeviceMemory;
 
     #[inline]
-    fn internal_object(&self) -> vk::DeviceMemory {
+    fn internal_object(&self) -> ash::vk::DeviceMemory {
         self.memory
     }
 }
@@ -627,8 +649,9 @@ impl Drop for DeviceMemory {
     #[inline]
     fn drop(&mut self) {
         unsafe {
-            let vk = self.device.pointers();
-            vk.FreeMemory(self.device.internal_object(), self.memory, ptr::null());
+            let fns = self.device.fns();
+            fns.v1_0
+                .free_memory(self.device.internal_object(), self.memory, ptr::null());
             let mut allocation_count = self
                 .device
                 .allocation_count()
@@ -686,8 +709,9 @@ impl MappedDeviceMemory {
     pub fn unmap(self) -> DeviceMemory {
         unsafe {
             let device = self.memory.device();
-            let vk = device.pointers();
-            vk.UnmapMemory(device.internal_object(), self.memory.memory);
+            let fns = device.fns();
+            fns.v1_0
+                .unmap_memory(device.internal_object(), self.memory.memory);
         }
 
         self.memory
@@ -708,35 +732,39 @@ impl MappedDeviceMemory {
     ///   the `MappedDeviceMemory`.
     ///
     #[inline]
-    pub unsafe fn read_write<T: ?Sized>(&self, range: Range<usize>) -> CpuAccess<T>
+    pub unsafe fn read_write<T: ?Sized>(&self, range: Range<DeviceSize>) -> CpuAccess<T>
     where
         T: Content,
     {
-        let vk = self.memory.device().pointers();
+        let fns = self.memory.device().fns();
         let pointer = T::ref_from_ptr(
-            (self.pointer as usize + range.start) as *mut _,
-            range.end - range.start,
+            (self.pointer as usize + range.start as usize) as *mut _,
+            (range.end - range.start) as usize,
         )
         .unwrap(); // TODO: error
 
         if !self.coherent {
-            let range = vk::MappedMemoryRange {
-                sType: vk::STRUCTURE_TYPE_MAPPED_MEMORY_RANGE,
-                pNext: ptr::null(),
+            let range = ash::vk::MappedMemoryRange {
                 memory: self.memory.internal_object(),
-                offset: range.start as u64,
-                size: (range.end - range.start) as u64,
+                offset: range.start,
+                size: range.end - range.start,
+                ..Default::default()
             };
 
-            // TODO: check result?
-            vk.InvalidateMappedMemoryRanges(self.memory.device().internal_object(), 1, &range);
+            // TODO: return result instead?
+            check_errors(fns.v1_0.invalidate_mapped_memory_ranges(
+                self.memory.device().internal_object(),
+                1,
+                &range,
+            ))
+            .unwrap();
         }
 
         CpuAccess {
             pointer: pointer,
             mem: self,
             coherent: self.coherent,
-            range: range,
+            range,
         }
     }
 }
@@ -791,8 +819,8 @@ impl DeviceMemoryMapping {
     pub fn new(
         device: Arc<Device>,
         memory: Arc<DeviceMemory>,
-        offset: u64,
-        size: u64,
+        offset: DeviceSize,
+        size: DeviceSize,
         flags: u32,
     ) -> Result<DeviceMemoryMapping, DeviceMemoryAllocError> {
         // VUID-vkMapMemory-memory-00678: "memory must not be currently host mapped".
@@ -803,19 +831,19 @@ impl DeviceMemoryMapping {
         }
 
         // VUID-vkMapMemory-offset-00679: "offset must be less than the size of memory"
-        if size != vk::WHOLE_SIZE && offset >= memory.size() as u64 {
+        if size != ash::vk::WHOLE_SIZE && offset >= memory.size() {
             return Err(DeviceMemoryAllocError::SpecViolation(679));
         }
 
         // VUID-vkMapMemory-size-00680: "If size is not equal to VK_WHOLE_SIZE, size must be
         // greater than 0".
-        if size != vk::WHOLE_SIZE && size == 0 {
+        if size != ash::vk::WHOLE_SIZE && size == 0 {
             return Err(DeviceMemoryAllocError::SpecViolation(680));
         }
 
         // VUID-vkMapMemory-size-00681: "If size is not equal to VK_WHOLE_SIZE, size must be less
         // than or equal to the size of the memory minus offset".
-        if size != vk::WHOLE_SIZE && size > memory.size() as u64 - offset {
+        if size != ash::vk::WHOLE_SIZE && size > memory.size() - offset {
             return Err(DeviceMemoryAllocError::SpecViolation(681));
         }
 
@@ -847,15 +875,15 @@ impl DeviceMemoryMapping {
         // VUID-vkMapMemory-device-parameter, VUID-vkMapMemory-memory-parameter and
         // VUID-vkMapMemory-ppData-parameter satisfied via Vulkano internally.
 
-        let vk = device.pointers();
+        let fns = device.fns();
         let ptr = unsafe {
             let mut output = MaybeUninit::uninit();
-            check_errors(vk.MapMemory(
+            check_errors(fns.v1_0.map_memory(
                 device.internal_object(),
                 memory.memory,
                 0,
-                memory.size as vk::DeviceSize,
-                0, /* reserved flags */
+                memory.size,
+                ash::vk::MemoryMapFlags::empty(),
                 output.as_mut_ptr(),
             ))?;
             output.assume_init()
@@ -888,8 +916,9 @@ impl Drop for DeviceMemoryMapping {
         let mut mapped = self.memory.mapped.lock().expect("Poisoned mutex");
 
         unsafe {
-            let vk = self.device.pointers();
-            vk.UnmapMemory(self.device.internal_object(), self.memory.memory);
+            let fns = self.device.fns();
+            fns.v1_0
+                .unmap_memory(self.device.internal_object(), self.memory.memory);
         }
 
         *mapped = false;
@@ -903,7 +932,7 @@ pub struct CpuAccess<'a, T: ?Sized + 'a> {
     pointer: *mut T,
     mem: &'a MappedDeviceMemory,
     coherent: bool,
-    range: Range<usize>,
+    range: Range<DeviceSize>,
 }
 
 impl<'a, T: ?Sized + 'a> CpuAccess<'a, T> {
@@ -951,19 +980,22 @@ impl<'a, T: ?Sized + 'a> Drop for CpuAccess<'a, T> {
     fn drop(&mut self) {
         // If the memory doesn't have the `coherent` flag, we need to flush the data.
         if !self.coherent {
-            let vk = self.mem.as_ref().device().pointers();
+            let fns = self.mem.as_ref().device().fns();
 
-            let range = vk::MappedMemoryRange {
-                sType: vk::STRUCTURE_TYPE_MAPPED_MEMORY_RANGE,
-                pNext: ptr::null(),
+            let range = ash::vk::MappedMemoryRange {
                 memory: self.mem.as_ref().internal_object(),
-                offset: self.range.start as u64,
-                size: (self.range.end - self.range.start) as u64,
+                offset: self.range.start,
+                size: self.range.end - self.range.start,
+                ..Default::default()
             };
 
-            // TODO: check result?
             unsafe {
-                vk.FlushMappedMemoryRanges(self.mem.as_ref().device().internal_object(), 1, &range);
+                check_errors(fns.v1_0.flush_mapped_memory_ranges(
+                    self.mem.as_ref().device().internal_object(),
+                    1,
+                    &range,
+                ))
+                .unwrap();
             }
         }
     }
