@@ -8,22 +8,15 @@
 // according to those terms.
 
 use crate::check_errors;
-use crate::descriptor::descriptor::DescriptorDesc;
-use crate::descriptor::descriptor_set::UnsafeDescriptorSetLayout;
-use crate::descriptor::pipeline_layout::PipelineLayout;
-use crate::descriptor::pipeline_layout::PipelineLayoutAbstract;
-use crate::descriptor::pipeline_layout::PipelineLayoutCreationError;
-use crate::descriptor::pipeline_layout::PipelineLayoutDesc;
-use crate::descriptor::pipeline_layout::PipelineLayoutDescPcRange;
-use crate::descriptor::pipeline_layout::PipelineLayoutNotSupersetError;
-use crate::descriptor::pipeline_layout::PipelineLayoutSuperset;
-use crate::descriptor::pipeline_layout::PipelineLayoutSys;
+use crate::descriptor_set::layout::DescriptorSetLayout;
 use crate::device::Device;
 use crate::device::DeviceOwned;
 use crate::pipeline::cache::PipelineCache;
+use crate::pipeline::layout::PipelineLayout;
+use crate::pipeline::layout::PipelineLayoutCreationError;
+use crate::pipeline::layout::PipelineLayoutSupersetError;
 use crate::pipeline::shader::EntryPointAbstract;
 use crate::pipeline::shader::SpecializationConstants;
-use crate::vk;
 use crate::Error;
 use crate::OomError;
 use crate::SafeDeref;
@@ -42,69 +35,87 @@ use std::sync::Arc;
 /// The template parameter contains the descriptor set to use with this pipeline.
 ///
 /// All compute pipeline objects implement the `ComputePipelineAbstract` trait. You can turn any
-/// `Arc<ComputePipeline<Pl>>` into an `Arc<ComputePipelineAbstract>` if necessary.
+/// `Arc<ComputePipeline>` into an `Arc<ComputePipelineAbstract>` if necessary.
 ///
 /// Pass an optional `Arc` to a `PipelineCache` to enable pipeline caching. The vulkan
 /// implementation will handle the `PipelineCache` and check if it is available.
 /// Check the documentation of the `PipelineCache` for more information.
-pub struct ComputePipeline<Pl> {
+pub struct ComputePipeline {
     inner: Inner,
-    pipeline_layout: Pl,
+    pipeline_layout: Arc<PipelineLayout>,
 }
 
 struct Inner {
-    pipeline: vk::Pipeline,
+    pipeline: ash::vk::Pipeline,
     device: Arc<Device>,
 }
 
-impl ComputePipeline<()> {
+impl ComputePipeline {
     /// Builds a new `ComputePipeline`.
-    pub fn new<Cs>(
+    pub fn new<Cs, Css>(
         device: Arc<Device>,
         shader: &Cs,
-        specialization: &Cs::SpecializationConstants,
+        spec_constants: &Css,
         cache: Option<Arc<PipelineCache>>,
-    ) -> Result<ComputePipeline<PipelineLayout<Cs::PipelineLayout>>, ComputePipelineCreationError>
+    ) -> Result<ComputePipeline, ComputePipelineCreationError>
     where
-        Cs::PipelineLayout: Clone,
         Cs: EntryPointAbstract,
+        Css: SpecializationConstants,
     {
         unsafe {
-            let pipeline_layout = shader.layout().clone().build(device.clone())?;
+            let descriptor_set_layouts = shader
+                .descriptor_set_layout_descs()
+                .iter()
+                .map(|desc| {
+                    Ok(Arc::new(DescriptorSetLayout::new(
+                        device.clone(),
+                        desc.clone(),
+                    )?))
+                })
+                .collect::<Result<Vec<_>, OomError>>()?;
+            let pipeline_layout = Arc::new(PipelineLayout::new(
+                device.clone(),
+                descriptor_set_layouts,
+                shader.push_constant_range().iter().cloned(),
+            )?);
             ComputePipeline::with_unchecked_pipeline_layout(
                 device,
                 shader,
-                specialization,
+                spec_constants,
                 pipeline_layout,
                 cache,
             )
         }
     }
-}
 
-impl<Pl> ComputePipeline<Pl> {
     /// Builds a new `ComputePipeline` with a specific pipeline layout.
     ///
     /// An error will be returned if the pipeline layout isn't a superset of what the shader
     /// uses.
-    pub fn with_pipeline_layout<Cs>(
+    pub fn with_pipeline_layout<Cs, Css>(
         device: Arc<Device>,
         shader: &Cs,
-        specialization: &Cs::SpecializationConstants,
-        pipeline_layout: Pl,
+        spec_constants: &Css,
+        pipeline_layout: Arc<PipelineLayout>,
         cache: Option<Arc<PipelineCache>>,
-    ) -> Result<ComputePipeline<Pl>, ComputePipelineCreationError>
+    ) -> Result<ComputePipeline, ComputePipelineCreationError>
     where
-        Cs::PipelineLayout: Clone,
         Cs: EntryPointAbstract,
-        Pl: PipelineLayoutAbstract,
+        Css: SpecializationConstants,
     {
+        if Css::descriptors() != shader.spec_constants() {
+            return Err(ComputePipelineCreationError::IncompatibleSpecializationConstants);
+        }
+
         unsafe {
-            PipelineLayoutSuperset::ensure_superset_of(&pipeline_layout, shader.layout())?;
+            pipeline_layout.ensure_superset_of(
+                shader.descriptor_set_layout_descs(),
+                shader.push_constant_range(),
+            )?;
             ComputePipeline::with_unchecked_pipeline_layout(
                 device,
                 shader,
-                specialization,
+                spec_constants,
                 pipeline_layout,
                 cache,
             )
@@ -113,60 +124,57 @@ impl<Pl> ComputePipeline<Pl> {
 
     /// Same as `with_pipeline_layout`, but doesn't check whether the pipeline layout is a
     /// superset of what the shader expects.
-    pub unsafe fn with_unchecked_pipeline_layout<Cs>(
+    pub unsafe fn with_unchecked_pipeline_layout<Cs, Css>(
         device: Arc<Device>,
         shader: &Cs,
-        specialization: &Cs::SpecializationConstants,
-        pipeline_layout: Pl,
+        spec_constants: &Css,
+        pipeline_layout: Arc<PipelineLayout>,
         cache: Option<Arc<PipelineCache>>,
-    ) -> Result<ComputePipeline<Pl>, ComputePipelineCreationError>
+    ) -> Result<ComputePipeline, ComputePipelineCreationError>
     where
-        Cs::PipelineLayout: Clone,
         Cs: EntryPointAbstract,
-        Pl: PipelineLayoutAbstract,
+        Css: SpecializationConstants,
     {
-        let vk = device.pointers();
+        let fns = device.fns();
 
         let pipeline = {
-            let spec_descriptors = Cs::SpecializationConstants::descriptors();
-            let specialization = vk::SpecializationInfo {
-                mapEntryCount: spec_descriptors.len() as u32,
-                pMapEntries: spec_descriptors.as_ptr() as *const _,
-                dataSize: mem::size_of_val(specialization),
-                pData: specialization as *const Cs::SpecializationConstants as *const _,
+            let spec_descriptors = Css::descriptors();
+            let specialization = ash::vk::SpecializationInfo {
+                map_entry_count: spec_descriptors.len() as u32,
+                p_map_entries: spec_descriptors.as_ptr() as *const _,
+                data_size: mem::size_of_val(spec_constants),
+                p_data: spec_constants as *const Css as *const _,
             };
 
-            let stage = vk::PipelineShaderStageCreateInfo {
-                sType: vk::STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO,
-                pNext: ptr::null(),
-                flags: 0,
-                stage: vk::SHADER_STAGE_COMPUTE_BIT,
+            let stage = ash::vk::PipelineShaderStageCreateInfo {
+                flags: ash::vk::PipelineShaderStageCreateFlags::empty(),
+                stage: ash::vk::ShaderStageFlags::COMPUTE,
                 module: shader.module().internal_object(),
-                pName: shader.name().as_ptr(),
-                pSpecializationInfo: if specialization.dataSize == 0 {
+                p_name: shader.name().as_ptr(),
+                p_specialization_info: if specialization.data_size == 0 {
                     ptr::null()
                 } else {
                     &specialization
                 },
+                ..Default::default()
             };
 
-            let infos = vk::ComputePipelineCreateInfo {
-                sType: vk::STRUCTURE_TYPE_COMPUTE_PIPELINE_CREATE_INFO,
-                pNext: ptr::null(),
-                flags: 0,
-                stage: stage,
-                layout: PipelineLayoutAbstract::sys(&pipeline_layout).internal_object(),
-                basePipelineHandle: 0,
-                basePipelineIndex: 0,
+            let infos = ash::vk::ComputePipelineCreateInfo {
+                flags: ash::vk::PipelineCreateFlags::empty(),
+                stage,
+                layout: pipeline_layout.internal_object(),
+                base_pipeline_handle: ash::vk::Pipeline::null(),
+                base_pipeline_index: 0,
+                ..Default::default()
             };
 
             let cache_handle = match cache {
                 Some(ref cache) => cache.internal_object(),
-                None => vk::NULL_HANDLE,
+                None => ash::vk::PipelineCache::null(),
             };
 
             let mut output = MaybeUninit::uninit();
-            check_errors(vk.CreateComputePipelines(
+            check_errors(fns.v1_0.create_compute_pipelines(
                 device.internal_object(),
                 cache_handle,
                 1,
@@ -187,41 +195,40 @@ impl<Pl> ComputePipeline<Pl> {
     }
 }
 
-impl<Pl> fmt::Debug for ComputePipeline<Pl> {
+impl fmt::Debug for ComputePipeline {
     #[inline]
     fn fmt(&self, fmt: &mut fmt::Formatter) -> Result<(), fmt::Error> {
         write!(fmt, "<Vulkan compute pipeline {:?}>", self.inner.pipeline)
     }
 }
 
-impl<Pl> ComputePipeline<Pl> {
+impl ComputePipeline {
     /// Returns the `Device` this compute pipeline was created with.
     #[inline]
     pub fn device(&self) -> &Arc<Device> {
         &self.inner.device
     }
-
-    /// Returns the pipeline layout used in this compute pipeline.
-    #[inline]
-    pub fn layout(&self) -> &Pl {
-        &self.pipeline_layout
-    }
 }
 
 /// Trait implemented on all compute pipelines.
-pub unsafe trait ComputePipelineAbstract: PipelineLayoutAbstract {
+pub unsafe trait ComputePipelineAbstract: DeviceOwned {
     /// Returns an opaque object that represents the inside of the compute pipeline.
     fn inner(&self) -> ComputePipelineSys;
+
+    /// Returns the pipeline layout used in this compute pipeline.
+    fn layout(&self) -> &Arc<PipelineLayout>;
 }
 
-unsafe impl<Pl> ComputePipelineAbstract for ComputePipeline<Pl>
-where
-    Pl: PipelineLayoutAbstract,
-{
+unsafe impl ComputePipelineAbstract for ComputePipeline {
     #[inline]
     fn inner(&self) -> ComputePipelineSys {
         ComputePipelineSys(self.inner.pipeline, PhantomData)
     }
+
+    #[inline]
+    fn layout(&self) -> &Arc<PipelineLayout> {
+        &self.pipeline_layout
+    }
 }
 
 unsafe impl<T> ComputePipelineAbstract for T
@@ -233,84 +240,39 @@ where
     fn inner(&self) -> ComputePipelineSys {
         (**self).inner()
     }
+
+    #[inline]
+    fn layout(&self) -> &Arc<PipelineLayout> {
+        (**self).layout()
+    }
 }
 
 /// Opaque object that represents the inside of the compute pipeline. Can be made into a trait
 /// object.
 #[derive(Debug, Copy, Clone)]
-pub struct ComputePipelineSys<'a>(vk::Pipeline, PhantomData<&'a ()>);
+pub struct ComputePipelineSys<'a>(ash::vk::Pipeline, PhantomData<&'a ()>);
 
 unsafe impl<'a> VulkanObject for ComputePipelineSys<'a> {
-    type Object = vk::Pipeline;
-
-    const TYPE: vk::ObjectType = vk::OBJECT_TYPE_PIPELINE;
+    type Object = ash::vk::Pipeline;
 
     #[inline]
-    fn internal_object(&self) -> vk::Pipeline {
+    fn internal_object(&self) -> ash::vk::Pipeline {
         self.0
     }
 }
 
-unsafe impl<Pl> PipelineLayoutAbstract for ComputePipeline<Pl>
-where
-    Pl: PipelineLayoutAbstract,
-{
-    #[inline]
-    fn sys(&self) -> PipelineLayoutSys {
-        self.layout().sys()
-    }
-
-    #[inline]
-    fn descriptor_set_layout(&self, index: usize) -> Option<&Arc<UnsafeDescriptorSetLayout>> {
-        self.layout().descriptor_set_layout(index)
-    }
-}
-
-unsafe impl<Pl> PipelineLayoutDesc for ComputePipeline<Pl>
-where
-    Pl: PipelineLayoutDesc,
-{
-    #[inline]
-    fn num_sets(&self) -> usize {
-        self.pipeline_layout.num_sets()
-    }
-
-    #[inline]
-    fn num_bindings_in_set(&self, set: usize) -> Option<usize> {
-        self.pipeline_layout.num_bindings_in_set(set)
-    }
-
-    #[inline]
-    fn descriptor(&self, set: usize, binding: usize) -> Option<DescriptorDesc> {
-        self.pipeline_layout.descriptor(set, binding)
-    }
-
-    #[inline]
-    fn num_push_constants_ranges(&self) -> usize {
-        self.pipeline_layout.num_push_constants_ranges()
-    }
-
-    #[inline]
-    fn push_constants_range(&self, num: usize) -> Option<PipelineLayoutDescPcRange> {
-        self.pipeline_layout.push_constants_range(num)
-    }
-}
-
-unsafe impl<Pl> DeviceOwned for ComputePipeline<Pl> {
+unsafe impl DeviceOwned for ComputePipeline {
     #[inline]
     fn device(&self) -> &Arc<Device> {
         self.device()
     }
 }
 
-// TODO: remove in favor of ComputePipelineAbstract?
-unsafe impl<Pl> VulkanObject for ComputePipeline<Pl> {
-    type Object = vk::Pipeline;
-
-    const TYPE: vk::ObjectType = vk::OBJECT_TYPE_PIPELINE;
+unsafe impl VulkanObject for ComputePipeline {
+    type Object = ash::vk::Pipeline;
 
     #[inline]
-    fn internal_object(&self) -> vk::Pipeline {
+    fn internal_object(&self) -> ash::vk::Pipeline {
         self.inner.pipeline
     }
 }
@@ -319,8 +281,9 @@ impl Drop for Inner {
     #[inline]
     fn drop(&mut self) {
         unsafe {
-            let vk = self.device.pointers();
-            vk.DestroyPipeline(self.device.internal_object(), self.pipeline, ptr::null());
+            let fns = self.device.fns();
+            fns.v1_0
+                .destroy_pipeline(self.device.internal_object(), self.pipeline, ptr::null());
         }
     }
 }
@@ -333,7 +296,9 @@ pub enum ComputePipelineCreationError {
     /// Error while creating the pipeline layout object.
     PipelineLayoutCreationError(PipelineLayoutCreationError),
     /// The pipeline layout is not compatible with what the shader expects.
-    IncompatiblePipelineLayout(PipelineLayoutNotSupersetError),
+    IncompatiblePipelineLayout(PipelineLayoutSupersetError),
+    /// The provided specialization constants are not compatible with what the shader expects.
+    IncompatibleSpecializationConstants,
 }
 
 impl error::Error for ComputePipelineCreationError {
@@ -343,6 +308,7 @@ impl error::Error for ComputePipelineCreationError {
             ComputePipelineCreationError::OomError(ref err) => Some(err),
             ComputePipelineCreationError::PipelineLayoutCreationError(ref err) => Some(err),
             ComputePipelineCreationError::IncompatiblePipelineLayout(ref err) => Some(err),
+            ComputePipelineCreationError::IncompatibleSpecializationConstants => None,
         }
     }
 }
@@ -361,6 +327,9 @@ impl fmt::Display for ComputePipelineCreationError {
                 ComputePipelineCreationError::IncompatiblePipelineLayout(_) => {
                     "the pipeline layout is not compatible with what the shader expects"
                 }
+                ComputePipelineCreationError::IncompatibleSpecializationConstants => {
+                    "the provided specialization constants are not compatible with what the shader expects"
+                }
             }
         )
     }
@@ -380,9 +349,9 @@ impl From<PipelineLayoutCreationError> for ComputePipelineCreationError {
     }
 }
 
-impl From<PipelineLayoutNotSupersetError> for ComputePipelineCreationError {
+impl From<PipelineLayoutSupersetError> for ComputePipelineCreationError {
     #[inline]
-    fn from(err: PipelineLayoutNotSupersetError) -> ComputePipelineCreationError {
+    fn from(err: PipelineLayoutSupersetError) -> ComputePipelineCreationError {
         ComputePipelineCreationError::IncompatiblePipelineLayout(err)
     }
 }
@@ -408,18 +377,17 @@ mod tests {
     use crate::buffer::CpuAccessibleBuffer;
     use crate::command_buffer::AutoCommandBufferBuilder;
     use crate::command_buffer::CommandBufferUsage;
-    use crate::descriptor::descriptor::DescriptorBufferDesc;
-    use crate::descriptor::descriptor::DescriptorDesc;
-    use crate::descriptor::descriptor::DescriptorDescTy;
-    use crate::descriptor::descriptor::ShaderStages;
-    use crate::descriptor::descriptor_set::PersistentDescriptorSet;
-    use crate::descriptor::pipeline_layout::PipelineLayoutAbstract;
-    use crate::descriptor::pipeline_layout::PipelineLayoutDesc;
-    use crate::descriptor::pipeline_layout::PipelineLayoutDescPcRange;
+    use crate::descriptor_set::layout::DescriptorBufferDesc;
+    use crate::descriptor_set::layout::DescriptorDesc;
+    use crate::descriptor_set::layout::DescriptorDescTy;
+    use crate::descriptor_set::layout::DescriptorSetDesc;
+    use crate::descriptor_set::PersistentDescriptorSet;
     use crate::pipeline::shader::ShaderModule;
+    use crate::pipeline::shader::ShaderStages;
     use crate::pipeline::shader::SpecializationConstants;
     use crate::pipeline::shader::SpecializationMapEntry;
     use crate::pipeline::ComputePipeline;
+    use crate::pipeline::ComputePipelineAbstract;
     use crate::sync::now;
     use crate::sync::GpuFuture;
     use std::ffi::CStr;
@@ -477,45 +445,24 @@ mod tests {
         };
 
         let shader = unsafe {
-            #[derive(Debug, Copy, Clone)]
-            struct Layout;
-            unsafe impl PipelineLayoutDesc for Layout {
-                fn num_sets(&self) -> usize {
-                    1
-                }
-                fn num_bindings_in_set(&self, set: usize) -> Option<usize> {
-                    match set {
-                        0 => Some(1),
-                        _ => None,
-                    }
-                }
-                fn descriptor(&self, set: usize, binding: usize) -> Option<DescriptorDesc> {
-                    match (set, binding) {
-                        (0, 0) => Some(DescriptorDesc {
-                            ty: DescriptorDescTy::Buffer(DescriptorBufferDesc {
-                                dynamic: Some(false),
-                                storage: true,
-                            }),
-                            array_count: 1,
-                            stages: ShaderStages {
-                                compute: true,
-                                ..ShaderStages::none()
-                            },
-                            readonly: true,
-                        }),
-                        _ => None,
-                    }
-                }
-                fn num_push_constants_ranges(&self) -> usize {
-                    0
-                }
-                fn push_constants_range(&self, num: usize) -> Option<PipelineLayoutDescPcRange> {
-                    None
-                }
-            }
-
             static NAME: [u8; 5] = [109, 97, 105, 110, 0]; // "main"
-            module.compute_entry_point(CStr::from_ptr(NAME.as_ptr() as *const _), Layout)
+            module.compute_entry_point(
+                CStr::from_ptr(NAME.as_ptr() as *const _),
+                [DescriptorSetDesc::new([Some(DescriptorDesc {
+                    ty: DescriptorDescTy::Buffer(DescriptorBufferDesc {
+                        dynamic: Some(false),
+                        storage: true,
+                    }),
+                    array_count: 1,
+                    stages: ShaderStages {
+                        compute: true,
+                        ..ShaderStages::none()
+                    },
+                    readonly: true,
+                })])],
+                None,
+                SpecConsts::descriptors(),
+            )
         };
 
         #[derive(Debug, Copy, Clone)]
@@ -547,7 +494,7 @@ mod tests {
 
         let data_buffer =
             CpuAccessibleBuffer::from_data(device.clone(), BufferUsage::all(), false, 0).unwrap();
-        let layout = pipeline.layout().descriptor_set_layout(0).unwrap();
+        let layout = pipeline.layout().descriptor_set_layouts().get(0).unwrap();
         let set = PersistentDescriptorSet::start(layout.clone())
             .add_buffer(data_buffer.clone())
             .unwrap()
@@ -560,8 +507,7 @@ mod tests {
             CommandBufferUsage::OneTimeSubmit,
         )
         .unwrap();
-        cbb.dispatch([1, 1, 1], pipeline.clone(), set, (), vec![])
-            .unwrap();
+        cbb.dispatch([1, 1, 1], pipeline.clone(), set, ()).unwrap();
         let cb = cbb.build().unwrap();
 
         let future = now(device.clone())
