@@ -7,16 +7,15 @@
 // notice may not be copied, modified, or distributed except
 // according to those terms.
 
-use std::error;
-use std::fmt;
-use std::u32;
-
-use crate::descriptor::pipeline_layout::PipelineLayoutNotSupersetError;
 use crate::pipeline::input_assembly::PrimitiveTopology;
+use crate::pipeline::layout::PipelineLayoutSupersetError;
 use crate::pipeline::shader::ShaderInterfaceMismatchError;
 use crate::pipeline::vertex::IncompatibleVertexDefinitionError;
 use crate::Error;
 use crate::OomError;
+use std::error;
+use std::fmt;
+use std::u32;
 
 /// Error that can happen when creating a graphics pipeline.
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -25,31 +24,13 @@ pub enum GraphicsPipelineCreationError {
     OomError(OomError),
 
     /// The pipeline layout is not compatible with what the shaders expect.
-    IncompatiblePipelineLayout(PipelineLayoutNotSupersetError),
+    IncompatiblePipelineLayout(PipelineLayoutSupersetError),
 
-    /// The interface between the vertex shader and the geometry shader mismatches.
-    VertexGeometryStagesMismatch(ShaderInterfaceMismatchError),
+    /// The provided specialization constants are not compatible with what the shader expects.
+    IncompatibleSpecializationConstants,
 
-    /// The interface between the vertex shader and the tessellation control shader mismatches.
-    VertexTessControlStagesMismatch(ShaderInterfaceMismatchError),
-
-    /// The interface between the vertex shader and the fragment shader mismatches.
-    VertexFragmentStagesMismatch(ShaderInterfaceMismatchError),
-
-    /// The interface between the tessellation control shader and the tessellation evaluation
-    /// shader mismatches.
-    TessControlTessEvalStagesMismatch(ShaderInterfaceMismatchError),
-
-    /// The interface between the tessellation evaluation shader and the geometry shader
-    /// mismatches.
-    TessEvalGeometryStagesMismatch(ShaderInterfaceMismatchError),
-
-    /// The interface between the tessellation evaluation shader and the fragment shader
-    /// mismatches.
-    TessEvalFragmentStagesMismatch(ShaderInterfaceMismatchError),
-
-    /// The interface between the geometry shader and the fragment shader mismatches.
-    GeometryFragmentStagesMismatch(ShaderInterfaceMismatchError),
+    /// The output interface of one shader and the input interface of the next shader does not match.
+    ShaderStagesMismatch(ShaderInterfaceMismatchError),
 
     /// The output of the fragment shader is not compatible with what the render pass subpass
     /// expects.
@@ -62,38 +43,56 @@ pub enum GraphicsPipelineCreationError {
     /// has been exceeded.
     MaxVertexInputBindingStrideExceeded {
         /// Index of the faulty binding.
-        binding: usize,
+        binding: u32,
         /// Maximum allowed value.
-        max: usize,
+        max: u32,
         /// Value that was passed.
-        obtained: usize,
+        obtained: u32,
     },
 
     /// The maximum number of vertex sources has been exceeded.
     MaxVertexInputBindingsExceeded {
         /// Maximum allowed value.
-        max: usize,
+        max: u32,
         /// Value that was passed.
-        obtained: usize,
+        obtained: u32,
     },
 
     /// The maximum offset for a vertex attribute has been exceeded. This means that your vertex
     /// struct is too large.
     MaxVertexInputAttributeOffsetExceeded {
         /// Maximum allowed value.
-        max: usize,
+        max: u32,
         /// Value that was passed.
-        obtained: usize,
+        obtained: u32,
     },
 
     /// The maximum number of vertex attributes has been exceeded.
     MaxVertexInputAttributesExceeded {
         /// Maximum allowed value.
-        max: usize,
+        max: u32,
         /// Value that was passed.
         obtained: usize,
     },
 
+    /// The `vertex_attribute_instance_rate_divisor` feature must be enabled in order to use
+    /// instance rate divisors.
+    VertexAttributeInstanceRateDivisorFeatureNotEnabled,
+
+    /// The `vertex_attribute_instance_rate_zero_divisor` feature must be enabled in order to use
+    /// an instance rate divisor of zero.
+    VertexAttributeInstanceRateZeroDivisorFeatureNotEnabled,
+
+    /// The maximum value for the instance rate divisor has been exceeded.
+    MaxVertexAttribDivisorExceeded {
+        /// Index of the faulty binding.
+        binding: u32,
+        /// Maximum allowed value.
+        max: u32,
+        /// Value that was passed.
+        obtained: u32,
+    },
+
     /// The user requested to use primitive restart, but the primitive topology doesn't support it.
     PrimitiveDoesntSupportPrimitiveRestart {
         /// The topology that doesn't support primitive restart.
@@ -182,6 +181,12 @@ pub enum GraphicsPipelineCreationError {
 
     /// The `alpha_to_one` feature must be enabled in order to use alpha-to-one.
     AlphaToOneFeatureNotEnabled,
+
+    /// The device doesn't support using the `multiview´ feature with geometry shaders.
+    MultiviewGeometryShaderNotSupported,
+
+    /// The device doesn't support using the `multiview´ feature with tessellation shaders.
+    MultiviewTessellationShaderNotSupported,
 }
 
 impl error::Error for GraphicsPipelineCreationError {
@@ -190,13 +195,7 @@ impl error::Error for GraphicsPipelineCreationError {
         match *self {
             GraphicsPipelineCreationError::OomError(ref err) => Some(err),
             GraphicsPipelineCreationError::IncompatiblePipelineLayout(ref err) => Some(err),
-            GraphicsPipelineCreationError::VertexGeometryStagesMismatch(ref err) => Some(err),
-            GraphicsPipelineCreationError::VertexTessControlStagesMismatch(ref err) => Some(err),
-            GraphicsPipelineCreationError::VertexFragmentStagesMismatch(ref err) => Some(err),
-            GraphicsPipelineCreationError::TessControlTessEvalStagesMismatch(ref err) => Some(err),
-            GraphicsPipelineCreationError::TessEvalGeometryStagesMismatch(ref err) => Some(err),
-            GraphicsPipelineCreationError::TessEvalFragmentStagesMismatch(ref err) => Some(err),
-            GraphicsPipelineCreationError::GeometryFragmentStagesMismatch(ref err) => Some(err),
+            GraphicsPipelineCreationError::ShaderStagesMismatch(ref err) => Some(err),
             GraphicsPipelineCreationError::IncompatibleVertexDefinition(ref err) => Some(err),
             _ => None,
         }
@@ -212,34 +211,15 @@ impl fmt::Display for GraphicsPipelineCreationError {
             "{}",
             match *self {
                 GraphicsPipelineCreationError::OomError(_) => "not enough memory available",
-                GraphicsPipelineCreationError::VertexGeometryStagesMismatch(_) => {
-                    "the interface between the vertex shader and the geometry shader mismatches"
-                }
-                GraphicsPipelineCreationError::VertexTessControlStagesMismatch(_) => {
-                    "the interface between the vertex shader and the tessellation control shader \
-                 mismatches"
-                }
-                GraphicsPipelineCreationError::VertexFragmentStagesMismatch(_) => {
-                    "the interface between the vertex shader and the fragment shader mismatches"
-                }
-                GraphicsPipelineCreationError::TessControlTessEvalStagesMismatch(_) => {
-                    "the interface between the tessellation control shader and the tessellation \
-                 evaluation shader mismatches"
-                }
-                GraphicsPipelineCreationError::TessEvalGeometryStagesMismatch(_) => {
-                    "the interface between the tessellation evaluation shader and the geometry \
-                 shader mismatches"
-                }
-                GraphicsPipelineCreationError::TessEvalFragmentStagesMismatch(_) => {
-                    "the interface between the tessellation evaluation shader and the fragment \
-                 shader mismatches"
-                }
-                GraphicsPipelineCreationError::GeometryFragmentStagesMismatch(_) => {
-                    "the interface between the geometry shader and the fragment shader mismatches"
+                GraphicsPipelineCreationError::ShaderStagesMismatch(_) => {
+                    "the output interface of one shader and the input interface of the next shader does not match"
                 }
                 GraphicsPipelineCreationError::IncompatiblePipelineLayout(_) => {
                     "the pipeline layout is not compatible with what the shaders expect"
                 }
+                GraphicsPipelineCreationError::IncompatibleSpecializationConstants => {
+                    "the provided specialization constants are not compatible with what the shader expects"
+                }
                 GraphicsPipelineCreationError::FragmentShaderRenderPassIncompatible => {
                     "the output of the fragment shader is not compatible with what the render pass \
                  subpass expects"
@@ -260,6 +240,15 @@ impl fmt::Display for GraphicsPipelineCreationError {
                 GraphicsPipelineCreationError::MaxVertexInputAttributesExceeded { .. } => {
                     "the maximum number of vertex attributes has been exceeded"
                 }
+                GraphicsPipelineCreationError::VertexAttributeInstanceRateDivisorFeatureNotEnabled => {
+                    "the `vertex_attribute_instance_rate_divisor` feature must be enabled in order to use instance rate divisors"
+                }
+                GraphicsPipelineCreationError::VertexAttributeInstanceRateZeroDivisorFeatureNotEnabled => {
+                    "the `vertex_attribute_instance_rate_zero_divisor` feature must be enabled in order to use an instance rate divisor of zero"
+                }
+                GraphicsPipelineCreationError::MaxVertexAttribDivisorExceeded { .. } => {
+                    "the maximum value for the instance rate divisor has been exceeded"
+                }
                 GraphicsPipelineCreationError::PrimitiveDoesntSupportPrimitiveRestart {
                     ..
                 } => {
@@ -343,6 +332,12 @@ impl fmt::Display for GraphicsPipelineCreationError {
                 GraphicsPipelineCreationError::AlphaToOneFeatureNotEnabled => {
                     "the `alpha_to_one` feature must be enabled in order to use alpha-to-one"
                 }
+                GraphicsPipelineCreationError::MultiviewGeometryShaderNotSupported => {
+                    "the device doesn't support using the `multiview´ feature with geometry shaders"
+                }
+                GraphicsPipelineCreationError::MultiviewTessellationShaderNotSupported => {
+                    "the device doesn't support using the `multiview´ feature with tessellation shaders"
+                }
             }
         )
     }
@@ -355,9 +350,9 @@ impl From<OomError> for GraphicsPipelineCreationError {
     }
 }
 
-impl From<PipelineLayoutNotSupersetError> for GraphicsPipelineCreationError {
+impl From<PipelineLayoutSupersetError> for GraphicsPipelineCreationError {
     #[inline]
-    fn from(err: PipelineLayoutNotSupersetError) -> GraphicsPipelineCreationError {
+    fn from(err: PipelineLayoutSupersetError) -> GraphicsPipelineCreationError {
         GraphicsPipelineCreationError::IncompatiblePipelineLayout(err)
     }
 }
