@@ -15,7 +15,6 @@ use crate::render_pass::ensure_image_view_compatible;
 use crate::render_pass::AttachmentsList;
 use crate::render_pass::IncompatibleRenderPassAttachmentError;
 use crate::render_pass::RenderPass;
-use crate::vk;
 use crate::Error;
 use crate::OomError;
 use crate::SafeDeref;
@@ -77,7 +76,7 @@ use std::sync::Arc;
 pub struct Framebuffer<A> {
     device: Arc<Device>,
     render_pass: Arc<RenderPass>,
-    framebuffer: vk::Framebuffer,
+    framebuffer: ash::vk::Framebuffer,
     dimensions: [u32; 3],
     resources: A,
 }
@@ -121,7 +120,7 @@ impl Framebuffer<()> {
 /// Prototype of a framebuffer.
 pub struct FramebufferBuilder<A> {
     render_pass: Arc<RenderPass>,
-    raw_ids: SmallVec<[vk::ImageView; 8]>,
+    raw_ids: SmallVec<[ash::vk::ImageView; 8]>,
     dimensions: FramebufferBuilderDimensions,
     attachments: A,
 }
@@ -281,34 +280,54 @@ where
 
         // Checking the dimensions against the limits.
         {
-            let limits = device.physical_device().limits();
+            let properties = device.physical_device().properties();
             let limits = [
-                limits.max_framebuffer_width(),
-                limits.max_framebuffer_height(),
-                limits.max_framebuffer_layers(),
+                properties.max_framebuffer_width,
+                properties.max_framebuffer_height,
+                properties.max_framebuffer_layers,
             ];
             if dimensions[0] > limits[0] || dimensions[1] > limits[1] || dimensions[2] > limits[2] {
                 return Err(FramebufferCreationError::DimensionsTooLarge);
             }
         }
 
-        let framebuffer = unsafe {
-            let vk = device.pointers();
+        let mut layers = 1;
 
-            let infos = vk::FramebufferCreateInfo {
-                sType: vk::STRUCTURE_TYPE_FRAMEBUFFER_CREATE_INFO,
-                pNext: ptr::null(),
-                flags: 0, // reserved
-                renderPass: self.render_pass.inner().internal_object(),
-                attachmentCount: self.raw_ids.len() as u32,
-                pAttachments: self.raw_ids.as_ptr(),
+        if let Some(multiview) = self.render_pass.desc().multiview() {
+            // There needs to be at least as many layers in the framebuffer
+            // as the highest layer that gets referenced by the multiview masking.
+            if multiview.highest_used_layer() > dimensions[2] {
+                return Err(FramebufferCreationError::InsufficientLayerCount {
+                    minimum: multiview.highest_used_layer(),
+                    current: dimensions[2],
+                });
+            }
+
+            // VUID-VkFramebufferCreateInfo-renderPass-02531
+            // The framebuffer has to be created with one layer if multiview is enabled even though
+            // the underlying images generally have more layers
+            // but these layers get used by the multiview functionality.
+            if multiview.view_masks.iter().any(|&mask| mask != 0) {
+                layers = 1;
+            }
+        }
+
+        let framebuffer = unsafe {
+            let fns = device.fns();
+
+            let infos = ash::vk::FramebufferCreateInfo {
+                flags: ash::vk::FramebufferCreateFlags::empty(),
+                render_pass: self.render_pass.inner().internal_object(),
+                attachment_count: self.raw_ids.len() as u32,
+                p_attachments: self.raw_ids.as_ptr(),
                 width: dimensions[0],
                 height: dimensions[1],
-                layers: dimensions[2],
+                layers,
+                ..Default::default()
             };
 
             let mut output = MaybeUninit::uninit();
-            check_errors(vk.CreateFramebuffer(
+            check_errors(fns.v1_0.create_framebuffer(
                 device.internal_object(),
                 &infos,
                 ptr::null(),
@@ -465,23 +484,25 @@ impl<A> Drop for Framebuffer<A> {
     #[inline]
     fn drop(&mut self) {
         unsafe {
-            let vk = self.device.pointers();
-            vk.DestroyFramebuffer(self.device.internal_object(), self.framebuffer, ptr::null());
+            let fns = self.device.fns();
+            fns.v1_0.destroy_framebuffer(
+                self.device.internal_object(),
+                self.framebuffer,
+                ptr::null(),
+            );
         }
     }
 }
 
 /// Opaque object that represents the internals of a framebuffer.
 #[derive(Debug, Copy, Clone)]
-pub struct FramebufferSys<'a>(vk::Framebuffer, PhantomData<&'a ()>);
+pub struct FramebufferSys<'a>(ash::vk::Framebuffer, PhantomData<&'a ()>);
 
 unsafe impl<'a> VulkanObject for FramebufferSys<'a> {
-    type Object = vk::Framebuffer;
-
-    const TYPE: vk::ObjectType = vk::OBJECT_TYPE_FRAMEBUFFER;
+    type Object = ash::vk::Framebuffer;
 
     #[inline]
-    fn internal_object(&self) -> vk::Framebuffer {
+    fn internal_object(&self) -> ash::vk::Framebuffer {
         self.0
     }
 }
@@ -493,6 +514,15 @@ pub enum FramebufferCreationError {
     OomError(OomError),
     /// The requested dimensions exceed the device's limits.
     DimensionsTooLarge,
+    /// The number of minimum layers expected by the render pass exceed the framebuffer layers.
+    /// This can happen when the multiview feature is enabled and the specified view or correlation
+    /// masks refer to more layers than the framebuffer has.
+    InsufficientLayerCount {
+        /// Minimum number of layers.
+        minimum: u32,
+        /// Number of framebuffer layers.
+        current: u32,
+    },
     /// The attachment has a size that isn't compatible with the requested framebuffer dimensions.
     AttachmentDimensionsIncompatible {
         /// Expected dimensions.
@@ -542,6 +572,9 @@ impl fmt::Display for FramebufferCreationError {
                 FramebufferCreationError::DimensionsTooLarge => {
                     "the dimensions of the framebuffer are too large"
                 }
+                FramebufferCreationError::InsufficientLayerCount { .. } => {
+                    "the number of minimum layers expected by the render pass exceed the framebuffer layers"
+                }
                 FramebufferCreationError::AttachmentDimensionsIncompatible { .. } => {
                     "the attachment has a size that isn't compatible with the framebuffer dimensions"
                 }
